package org.intranet.games.minesweeper.commandline;

import org.intranet.games.minesweeper.*;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

/** 
 *
 * @author Chris Dailey
 */
public final class MineSweeper
{
  public MineSweeper()
  {
  }

  public void play()
  {
    pf.init();
    InputStreamReader isr = new InputStreamReader(System.in);
    BufferedReader br = new BufferedReader(isr);
    String input;
    try {
      printPlayingField(pf);
      while ((input = br.readLine()) != null)
      {
        if ("exit".equals(input)) break;
        char c = input.charAt(0);
        if (c != 'm' && c != 'c')
        {
          System.err.println("did not understand input '" + input + "'");
          continue;
        }
        Square s = findSquare(br);
        if (s == null) continue;
        if (c == 'm') s.mark();
        else if (c == 'c') s.uncover();
        else System.err.println("did not understand input " + input);
        printPlayingField(pf);
      }
    } catch(java.io.IOException ioe) {
      ioe.printStackTrace();
    }
    System.out.println("Exitting");
  }

  public static final void printPlayingField(PlayingField pf)
  {
    System.out.print("   ");
    for (int x = 0; x < pf.getWidth(); x++)
      System.out.print(" "+(x+1)+"  ");
    System.out.println();
    for (int y = 0; y < pf.getHeight(); y++)
    {
      System.out.print((y+1) + ":");
      for (int x = 0; x < pf.getWidth(); x++)
      {
        Square s = pf.getSquare(x, y);
        Square.State state = s.getState();
        if (state == Square.State.covered)
          System.out.print(" [C]");
        else if (state == Square.State.question)
          System.out.print(" [?]");
        else if (state == Square.State.flagged)
          System.out.print(" [F]");
        else
        {
          if (s.hasMine())
          {
            System.out.print(" XXX");
            continue;
          }
          int n = s.getNumber();
          System.out.print(" [" + ((n == 0) ? ' ' : (char)(n+'0')) + "]");
          continue;
        }
      }
      System.out.println();
    }
    System.out.println("m=mark, c=clear, exit");
  }

  public static void main(String[] args)
  {
    new MineSweeper().play();
  }

  private Square findSquare(BufferedReader br)
  {
    System.out.println("Enter number,number  ( e.g. 5,5 )");
    String input;
    try {
      input = br.readLine();
    }
    catch (IOException ioe)
    {
      System.err.println("got error " + ioe.getMessage());
      return null;
    }
    int comma = input.indexOf(',');
    if (comma < 0)
    {
      System.err.println("no comma found in '" + input + "'");
      return null;
    }
    int column = Integer.parseInt(input.substring(0, comma)) - 1;
    int row = Integer.parseInt(input.substring(comma+1)) - 1;
    if (column < 0 || column >= pf.getWidth() ||
        row < 0 || row >= pf.getHeight())
    {
      System.err.println("Illegal column or row.");
      return null;
    }
    return pf.getSquare(column, row);
  }

  private PlayingField pf = new PlayingField();
}