package org.intranet.games.minesweeper.swing;
// Copyright 2002 Chris Dailey
// License terms at http://www.gnu.org/licenses/lgpl.html

/** 
* A graphical component that looks like an LED representing a standard digit.
* It has a black background, its LEDs are by default green.  Off fields
* can be a percentage of the on fields.
* @author Chris Dailey
*/
public class LED
  extends javax.swing.JComponent
{
  public static final int numLights = 7;
  private static final int lineMaxX = 4;
  private static final int lineMaxY = 7;
  private static boolean power = true;
  private static final java.awt.Point[] starts = {
    new java.awt.Point(1,0),
    new java.awt.Point(0,1),
    new java.awt.Point(3,1),
    new java.awt.Point(1,3),
    new java.awt.Point(0,4),
    new java.awt.Point(3,4),
    new java.awt.Point(1,6)
  };
  private java.awt.Point[] ends = {
    new java.awt.Point(2,0),
    new java.awt.Point(0,2),
    new java.awt.Point(3,2),
    new java.awt.Point(2,3),
    new java.awt.Point(0,5),
    new java.awt.Point(3,5),
    new java.awt.Point(2,6)
  };

  private java.awt.Insets insets = new java.awt.Insets(3,3,3,3);
  public java.awt.Insets getInsets() { return insets; }

  public void setBackground(java.awt.Color c)
  {
    super.setBackground(c);
    java.awt.Color fg = getForeground();
    if (fg != null)
      dimColor = getAverageColor(c, fg, bkgPct);
    repaint();
  }
  public void setForeground(java.awt.Color c)
  {
    super.setForeground(c);
    java.awt.Color bg = getBackground();
    if (bg != null)
      dimColor = getAverageColor(bg, c, bkgPct);
  }
  java.awt.Color dimColor;
  private final float bkgPct = 0.75F;

  private boolean[] lights = new boolean[numLights];
  public LED()
  {
    super();
    setPreferredSize(new java.awt.Dimension(23, 36));
    setBackground(java.awt.Color.black);
    setForeground(java.awt.Color.green);
  }

  private static java.awt.Color getAverageColor(
    java.awt.Color bg,
    java.awt.Color fg,
    float bgPct)
  {
    // TODO : bgPct must be btw 0 & 1
    float fgPct = 1.0F - bgPct;
    java.awt.Color avg = new java.awt.Color(
       (int)(bg.getRed()*bgPct   + fg.getRed()*fgPct),
       (int)(bg.getGreen()*bgPct + fg.getGreen()*fgPct),
       (int)(bg.getBlue()*bgPct  + fg.getBlue()*fgPct));
    return avg;
  }
  public void paintComponent(java.awt.Graphics g)
  {
    java.awt.Rectangle paintRect = getBounds();
    java.awt.Color bg = getBackground();
    g.setColor(bg);
    g.fillRect(paintRect.x, paintRect.y, paintRect.width, paintRect.height);

    int availableWidth = paintRect.width - insets.left - insets.right;
    int availableHeight = paintRect.height - insets.top - insets.bottom;
    float xRatio = availableWidth * 1.0F / lineMaxX;
    float yRatio = availableHeight * 1.0F / lineMaxY;
    java.awt.Color ledColor = getForeground();
    for (int i = 0; i < numLights; i++)
    {
      g.setColor((lights[i] && power)?ledColor:dimColor);
      java.awt.Point start = starts[i];
      java.awt.Point end = ends[i];
      int x1 = (int)((start.x + 0.5) * xRatio - xRatio/2.5);
      int y1 = (int)((start.y + 0.5) * yRatio - yRatio/2.5);
      int w = (int)((end.x + 0.5) * xRatio + xRatio/2.5) - x1;
      int h = (int)((end.y + 0.5) * yRatio + yRatio/2.5) - y1;
      g.fillRect(x1 + insets.left, y1 + insets.top, w, h);
    }
  }

  public void setLights(boolean[] newLights)
  {
    for (int i = 0; i < numLights; i++)
      lights[i] = newLights[i];
    repaint();
  }
  public void setLight(int index, boolean value)
  {
    lights[index] = value;
    repaint();
  }

  public boolean getPower() { return power; }
  public void setPower(boolean onOff)
  {
    if (power != onOff)
    {
      power = onOff;
      repaint();
    }
  }
}