package org.intranet.games.minesweeper.swing;
// Copyright 2002 Chris Dailey
// License terms at http://www.gnu.org/licenses/lgpl.html

import javax.swing.JComponent;
import java.awt.BorderLayout;
import java.awt.event.MouseAdapter;

import org.intranet.games.minesweeper.PlayingField;
import org.intranet.games.minesweeper.Square;
import org.intranet.games.minesweeper.Timer;
import org.intranet.graphics.zing.Grid;

/** 
*
* @author Chris Dailey
*/
class PlayingFieldView
  extends JComponent
  implements PlayingField.PlayingFieldListener
{
  private PlayingField playingField;
  private NumberDisplay flagCounter = new NumberDisplay(3);
  private TimerDisplay timerDisplay = new TimerDisplay(3);
  private GameStatusButton restart = new GameStatusButton();
  private Grid mGrid;
  private SquareRenderer mSquareRenderer;

  class TimerDisplay extends NumberDisplay implements Timer.TimerListener
  {
    TimerDisplay(int value) { super(value); }
    public void timerValueChanged(int value) { setValue(value); }
  }

  PlayingFieldView(PlayingField pf)
  {
    super();
    playingField = pf;
    setLayout(new BorderLayout());
    pf.addTimerListener(timerDisplay);
    pf.addPlayingFieldListener(this);

    javax.swing.JPanel topBar = new javax.swing.JPanel();
    topBar.setLayout(new BorderLayout());

    topBar.add(flagCounter, BorderLayout.WEST);

    restart.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent ae) {
        playingField.init();
      }
    });
    javax.swing.JPanel restartPanel = new javax.swing.JPanel();
    restartPanel.add(restart);
    topBar.add(restartPanel, BorderLayout.CENTER);

    topBar.add(timerDisplay, BorderLayout.EAST);

    add(topBar, BorderLayout.NORTH);

    PlayingFieldModel pfm = new PlayingFieldModel(playingField);
    mSquareRenderer = new SquareRenderer(pfm);
    mGrid = new Grid(pfm, mSquareRenderer);
    add(mGrid, BorderLayout.CENTER);
    mGrid.addMouseListener(new MouseAdapter() {
      Square threatenedSquare;
      public void mouseClicked(java.awt.event.MouseEvent me) {
        Square s = (Square)mGrid.getCellValueAtMouseLocation(me);
        if ((me.getModifiers() & me.BUTTON1_MASK) != 0)
          s.uncover();
        else if ((me.getModifiers() & me.BUTTON3_MASK) != 0)
          s.mark();
      }
      public void mousePressed(java.awt.event.MouseEvent me) {
        if (threatenedSquare != null)
        {
          threatenedSquare.unthreaten();
          threatenedSquare = null;
        }
        Square s = (Square)mGrid.getCellValueAtMouseLocation(me);
        if ((me.getModifiers() & me.BUTTON1_MASK) != 0 &&
            s.getState() == Square.State.covered)
        {
          Square selectedSquare = mSquareRenderer.getSelectedSquare();
          mSquareRenderer.setSelectedSquare(s);
          if (selectedSquare != null)
            repaintSquare(selectedSquare);
          repaintSquare(s);
          return;
        }
        if (s.getState() == Square.State.uncovered &&
            (me.getModifiers() & me.BUTTON3_MASK) != 0)
        {
          threatenedSquare = s;
          threatenedSquare.threaten();
        }
      }
      public void mouseReleased(java.awt.event.MouseEvent me) {
        if (threatenedSquare != null)
        {
          threatenedSquare.unthreaten();
          threatenedSquare = null;
        }
        if ((me.getModifiers() & me.BUTTON1_MASK) != 0)
        {
          Square selectedSquare = mSquareRenderer.getSelectedSquare();
          if (selectedSquare != null)
          {
            mSquareRenderer.setSelectedSquare(null);
            repaintSquare(selectedSquare);
          }
        }
      }
    });
  }

  private void repaintSquare(Square s)
  {
    int cellWidth = mSquareRenderer.getWidth();
    int cellHeight = mSquareRenderer.getHeight();
    mGrid.repaint(cellWidth * s.getX(), cellHeight * s.getY(),
      cellWidth, cellHeight);
  }

  public void playingFieldConfigurationChanged(
    PlayingField.PlayingFieldEvent pfe)
  {
  }

  public void playingFieldCounterUpdated(int value)
  {
    flagCounter.setValue(value);
    if (playingField.getStatus() != PlayingField.Status.initted)
      flagCounter.setPower(false);
  }
  public void playingFieldStatusChanged(PlayingField.Status newStatus)
  {
    restart.setStatus(newStatus);
    if (newStatus == PlayingField.Status.initted)
    {
      flagCounter.setValue(Integer.MAX_VALUE);
      timerDisplay.setValue(Integer.MAX_VALUE);
    }
    else if (newStatus == PlayingField.Status.inprogress)
    {
      flagCounter.setPower(true);
    }
  }
  public void playingFieldSquareChanged(Square s)
  {
  }
}