package org.intranet.games.minesweeper.swing;
// Copyright 2002 Chris Dailey
// License terms at http://www.gnu.org/licenses/lgpl.html

import java.awt.Component;
import java.awt.Dimension;
import javax.swing.JLabel;
import javax.swing.border.Border;
import javax.swing.BorderFactory;

import org.intranet.graphics.zing.Grid;
import org.intranet.games.minesweeper.swing.PlayingFieldModel;
import org.intranet.games.minesweeper.Square;

/**
*
* @author Chris Dailey
*/
public class SquareRenderer
  extends JLabel
  implements Grid.GridCellRenderer
{
  private Square s;
  private Square mSelectedSquare;
  private PlayingFieldModel pfm;
  private static final Border raisedBorder =
    BorderFactory.createRaisedBevelBorder();
  private static final Border loweredBorder =
    BorderFactory.createLoweredBevelBorder();
  private static final Border lineBorder =
    BorderFactory.createLineBorder(java.awt.Color.gray, 1);
  private final java.awt.Color background;
  private final Dimension mPrefSize = new Dimension(25, 25);

  public SquareRenderer(PlayingFieldModel aPlayingFieldModel)
  {
    super();
    setHorizontalAlignment(CENTER);
    background = getBackground();
    pfm = aPlayingFieldModel;
  }

  public Dimension getPreferredSize()
  {
    return mPrefSize;
  }

  public Component getGridCellRenderer(int row, int col)
  {
    s = (Square)pfm.getValueAt(row, col);
    Square.State state = s.getState();
    setBackground(background);
    setOpaque(false);
    if (state == Square.State.covered)
    {
      setBorder((s == mSelectedSquare) ? loweredBorder : raisedBorder);
      setText("");
    }
    else if (state == Square.State.coveredEnd)
    {
      setBorder(raisedBorder);
      setText(s.hasMine() ? "O" : "");
    }
    else if (state == Square.State.coveredFlagEnd)
    {
      setBorder(raisedBorder);
      setText(s.hasMine() ? "F" : "X");
    }
    else if (state == Square.State.flagged)
    {
      setBorder(raisedBorder);
      setText("F");
    }
    else if (state == Square.State.question)
    {
      setBorder(raisedBorder);
      setText("?");
    }
    else if (state == Square.State.scared)
    {
      setBorder(loweredBorder);
      setText("");
    }
    else if (state == Square.State.threatened ||
      state == Square.State.uncovered ||
      state == Square.State.uncoveredEnd)
    {
      setBorder(lineBorder);
      if (s.hasMine())
      {
        setOpaque(true);
        setBackground(java.awt.Color.red);
        setText("O");
      }
      else
      {
        int number = s.getNumber();
        if (number == 0)
          setText(" ");
        else
          setText(Integer.toString(number));
      }
    }
    return this;
  }

  void setSelectedSquare(Square aSquare) { mSelectedSquare = aSquare; }
  Square getSelectedSquare() { return mSelectedSquare; }
}