package org.intranet.graphics.zing;
// Copyright 2002 Chris Dailey
// License terms at http://www.gnu.org/licenses/lgpl.html

import java.awt.Component;
import java.awt.Rectangle;
import javax.swing.JComponent;

/**
*
* @author Chris Dailey
*/
public class Grid
  extends JComponent
{
  private GridModel mGridModel;
  private GridCellRenderer mGridCellRenderer;
  public Grid(GridModel aGridModel, GridCellRenderer aGridCellRenderer)
  {
    super();
    mGridModel = aGridModel;
    mGridCellRenderer = aGridCellRenderer;
    aGridModel.addGridModelListener(new GridModelListener() {
      public void gridModelCellChanged(int row, int column)
      {
        Component renderer = mGridCellRenderer.getGridCellRenderer(0, 0);
        java.awt.Dimension dim = renderer.getPreferredSize();
        int width = (int)dim.getWidth();
        int height = (int)dim.getHeight();
        repaint(width * column, height * row, width, height);
      }
      public void gridModelConfigurationChanged()
      {
        repaint();
      }
    });
  }
  public final java.awt.Dimension getPreferredSize()
  {
    Component renderer = mGridCellRenderer.getGridCellRenderer(0, 0);
    java.awt.Dimension dim = renderer.getPreferredSize();
    return new java.awt.Dimension(
      (int)dim.getWidth() * mGridModel.getColumnCount(),
      (int)dim.getHeight() * mGridModel.getRowCount());
  }
  public Object getCellValueAtMouseLocation(java.awt.event.MouseEvent me)
  {
    Component renderer = mGridCellRenderer.getGridCellRenderer(0, 0);
    java.awt.Dimension dim = renderer.getPreferredSize();
    int row = me.getY() / (int)dim.getHeight();
    int col = me.getX() / (int)dim.getWidth();
    return mGridModel.getValueAt(row, col);
  }

  public interface GridCellRenderer
  {
    public Component getGridCellRenderer(int row, int col);
  }
  public interface GridModel
  {
    public int getRowCount();
    public int getColumnCount();
    public Object getValueAt(int row, int col);
    public void addGridModelListener(GridModelListener gml);
    public void removeGridModelListener(GridModelListener gml);
  }
  public interface GridModelListener
  {
    public void gridModelConfigurationChanged();
    public void gridModelCellChanged(int row, int column);
  }
  public final void paintComponent(java.awt.Graphics g)
  {
    int rows = mGridModel.getRowCount();
    int cols = mGridModel.getColumnCount();

    Component c = mGridCellRenderer.getGridCellRenderer(0, 0);
    java.awt.Dimension dim = c.getPreferredSize();
    int cellWidth = (int)dim.getWidth();
    int cellHeight = (int)dim.getHeight();
    Rectangle rect = g.getClipBounds();
    for (int row = 0; row < rows; row++)
    {
      for (int col = 0; col < cols; col++)
      {
        c = mGridCellRenderer.getGridCellRenderer(row, col);
        int x = col * cellWidth;
        int y = row * cellHeight;
        if (!rect.intersects(x, y, cellWidth, cellHeight))
          continue;
        java.awt.Graphics g2 = g.create(x, y, cellWidth, cellHeight);
        c.setBounds(0, 0, cellWidth, cellHeight);
        c.paint(g2);
        g2.dispose();
      }
    }
  }
}